
#makefile for Finite State Playground
#LINUX VERSION
#(does not include man to html or debug versions)
#Mike Hammond, 1/07

#general variables
CC = gcc
LEX = lex
YACC = yacc
YFLAGS = -d

#variables for re program
RNAME = re
RFLAGS = -DLEXFLAG -ll -ly -lexpat
RSRC = automata.c complement.c lex.yy.c minFSA.c y.tab.c regex.c \
		rev.c ss.c stringtester.c xmlfsaparser.c

#variables for intersect program
INAME = intersect
IFLAGS = -lexpat
ISRC = intersect.c xmlfsaparser.c automata.c ss.c complement.c minFSA.c \
		distinct.c

#variables for union program
UNAME = union
UFLAGS = -lexpat
USRC = union.c xmlfsaparser.c automata.c ss.c minFSA.c distinct.c \
		rev.c

#variables for concatenate program
CNAME = concatenate
CFLAGS = -lexpat
CSRC = concatenate.c xmlfsaparser.c automata.c ss.c minFSA.c distinct.c

#variables for kleene program
KNAME = kleene
KFLAGS = -lexpat
KSRC = kleene.c xmlfsaparser.c automata.c ss.c minFSA.c

#variables for dictionary program
DNAME = dictionary
DSRC = dictionary.c automata.c ss.c

all : $(RNAME) $(INAME) $(UNAME) $(CNAME) $(KNAME) $(DNAME)

$(CNAME) : *.c *.h
	$(CC) $(CFLAGS) $(CSRC) -o ../bin/$(CNAME)

$(UNAME) : *.c *.h
	$(CC) $(UFLAGS) $(USRC) -o ../bin/$(UNAME)

$(INAME) : *.c *.h
	$(CC) $(IFLAGS) $(ISRC) -o ../bin/$(INAME)

$(KNAME) : *.c *.h
	$(CC) $(KFLAGS) $(KSRC) -o ../bin/$(KNAME)

$(DNAME) : *.c *.h
	$(CC) $(DSRC) -o ../bin/$(DNAME)

$(RNAME) : lex.yy.c y.tab.c y.tab.h *.c *.h
	$(CC) -o ../bin/$(RNAME) $(RSRC) $(RFLAGS)

lex.yy.c : $(RNAME).l
	$(LEX) $(RNAME).l

y.tab.c y.tab.h : $(RNAME).y
	$(YACC) $(YFLAGS) $(RNAME).y

veryclean : clean
	$(if $(wildcard ../bin/$(RNAME)),rm ../bin/$(RNAME))
	$(if $(wildcard ../bin/$(INAME)),rm ../bin/$(INAME))
	$(if $(wildcard ../bin/$(UNAME)),rm ../bin/$(UNAME))
	$(if $(wildcard ../bin/$(CNAME)),rm ../bin/$(CNAME))
	$(if $(wildcard ../bin/$(KNAME)),rm ../bin/$(KNAME))
	$(if $(wildcard ../bin/$(DNAME)),rm ../bin/$(DNAME))

clean :
	$(if $(wildcard y.tab.*),rm y.tab.*)
	$(if $(wildcard lex.yy.*),rm lex.yy.*)
	$(if $(wildcard *.[ois]),rm *.[ois])

